@@ -11,11 +11,11 @@
 #include <QLoggingCategory>
 #include <QQmlApplicationEngine>
 #include <QQmlContext>
-
 #include <QQuickStyle>
 #include <QQuickView>
 #include <QString>
 #include <QStringList>
+#include <QSysInfo>
 #include <QVariant>
 
 #ifdef Q_OS_ANDROID
@@ -48,24 +48,29 @@
 #include "models/feedsproxymodel.h"
 #include "models/podcastsearchmodel.h"
 #include "models/queuemodel.h"
-#include "mpris2/mpris2.h"
 #include "settingsmanager.h"
 #include "storagemanager.h"
 #include "sync/sync.h"
 #include "sync/syncutils.h"
 
+#ifdef Q_OS_WINDOWS
+#include <windows.h>
+#endif
+
 #ifdef Q_OS_ANDROID
 Q_DECL_EXPORT
 #endif
 
 int main(int argc, char *argv[])
 {
-    if (qEnvironmentVariableIsEmpty("QT_ENABLE_GLYPH_CACHE_WORKAROUND")) {
+    if (QSysInfo::currentCpuArchitecture().contains(QStringLiteral("arm")) && qEnvironmentVariableIsEmpty("QT_ENABLE_GLYPH_CACHE_WORKAROUND")) {
         qputenv("QT_ENABLE_GLYPH_CACHE_WORKAROUND", "1");
     }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QGuiApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
     QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
 
 #ifdef Q_OS_ANDROID
     QGuiApplication app(argc, argv);
@@ -130,7 +135,6 @@ int main(int argc, char *argv[])
     qmlRegisterType<FeedsProxyModel>("org.kde.kasts", 1, 0, "FeedsProxyModel");
     qmlRegisterType<QueueModel>("org.kde.kasts", 1, 0, "QueueModel");
     qmlRegisterType<EpisodeProxyModel>("org.kde.kasts", 1, 0, "EpisodeProxyModel");
-    qmlRegisterType<Mpris2>("org.kde.kasts", 1, 0, "Mpris2");
     qmlRegisterType<PodcastSearchModel>("org.kde.kasts", 1, 0, "PodcastSearchModel");
     qmlRegisterType<ChapterModel>("org.kde.kasts", 1, 0, "ChapterModel");
 
